      subroutine init
c-----------------------------------------------------------------------
c  initial condition for 3d simulations of the Parker instability
c  subroutine - bound1
c               bound2
c               rand
c-----------------------------------------------------------------------
      implicit none
c
      include 'mpif.h'
      include 'common.inc'
c
      integer i, j, k, ip
      real xmin, xmax, ymin, ymax, zmin, zmax
      real rand, seed(np), randnum, vrms
      real Ei0p
c-----------------------------------------------------------------------
c  parameters
c-----------------------------------------------------------------------
      nstep = 0
      t     = 0.
      dt    = 0.
c
      tend = 20.
      tdelta4 = 5.
      tdelta8 = 5.
c
      nunit4  = 0
      twrite4 = tdelta4
      nunit8  = 0
      twrite8 = tdelta8
c
      a = 1.
      cour = 0.6
c
      alpha = 1.
c
      xmin =  0.
      xmax = 12.
      ymin =  0.
      ymax = 12.
      zmin =  0.
      zmax = 12.
      dx = (xmax-xmin)/real(nx)
      dy = (ymax-ymin)/real(ny)
      dz = (zmax-zmin)/real(nz)
c
      iflag(0,1) = 1
      iflag(0,2) = 2
      iflag(0,3) = 3
c
      iflag(1,1) = 3
      iflag(1,2) = 2
      iflag(1,3) = 1
c
      iflag(2,1) = 2
      iflag(2,2) = 3
      iflag(2,3) = 1
c
      iflag(3,1) = 1
      iflag(3,2) = 3
      iflag(3,3) = 2
c
      iflag(4,1) = 3
      iflag(4,2) = 1
      iflag(4,3) = 2
c
      iflag(5,1) = 2
      iflag(5,2) = 1
      iflag(5,3) = 3
c
      randnum = 0.5
      vrms = 1.e-4
c-----------------------------------------------------------------------
c  initial condition
c-----------------------------------------------------------------------
      do 101 j=1,ny
         y(j) = ymin + (real(j)-0.5)*dy
 101  continue
c
      if (pi .eq. master) then
         do 102 ip=1,np
            seed(ip) = rand(randnum)
 102     continue
      endif
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call MPI_BCAST (seed,np,MPI_REAL,
     &                master,MPI_COMM_WORLD,err)
c
      do 103 k=1,nzp
      do 103 j=1,ny
      do 103 i=1,nx
         q1(i,j,k) = exp(-y(j))
         q2(i,j,k) = vrms*2.*sqrt(3.)*
     &               (rand(seed(pi+1))-0.5)*q1(i,j,k)
         q3(i,j,k) = vrms*2.*sqrt(3.)*
     &               (rand(seed(pi+1))-0.5)*q1(i,j,k)
         q4(i,j,k) = vrms*2.*sqrt(3.)*
     &               (rand(seed(pi+1))-0.5)*q1(i,j,k)
         q5(i,j,k) = sqrt(2.*alpha)*exp(-0.5*y(j))
         q6(i,j,k) = 0.
         q7(i,j,k) = 0.
 103  continue
c-----------------------------------------------------------------------
c  internal energy (or mass) of an initial state
c-----------------------------------------------------------------------
      Ei0p = 0.
c
      do 104 k=1,nzp
      do 104 j=1,ny
      do 104 i=1,nx
         Ei0p = Ei0p + q1(i,j,k)
 104  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call MPI_REDUCE (Ei0p,Ei0,1,MPI_REAL,
     &                 MPI_SUM,master,MPI_COMM_WORLD,err)
c
      if (pi .eq. master) then
         Ei0 = 1.5*Ei0
      endif
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound1
      call bound2
c-----------------------------------------------------------------------
c  end
c-----------------------------------------------------------------------
      return
      end
