      subroutine tvd3d
c-----------------------------------------------------------------------
c  3d isothermal mhd code for the simulations of the Parker instability
c  subroutines - mhdtvd
c                tstep
c                bound1
c                bound2
c-----------------------------------------------------------------------
      implicit none
c
      include 'mpif.h'
      include 'common.inc'
c
      integer i, j, k, nmod, ncount
      integer nsize, ip, pswap, ptag1, ptag2, idum1, idum2, ireq(2)
      real w1(nzp,ny,nzp,7), w2(nzp,ny,nzp,7)
c-----------------------------------------------------------------------
c  dt
c-----------------------------------------------------------------------
      nmod = mod(nstep,6)
      if (nstep.lt.6) call tstep
      if (nmod.eq.0)  call tstep
c-----------------------------------------------------------------------
c  order of integrations
c-----------------------------------------------------------------------
      ncount = 1
c
 204  continue
      if (ncount.ge.4) goto 205
      if (iflag(nmod,ncount).eq.1) then
         goto 201
      elseif (iflag(nmod,ncount).eq.2) then
         goto 202
      elseif (iflag(nmod,ncount).eq.3) then
         goto 203
      else
         goto 205
      endif
c-----------------------------------------------------------------------
c  integration along x
c-----------------------------------------------------------------------
 201  continue
c
      nd = nx
      dl = dx
c
      do 101 k=1,nzp
      do 101 j=1,ny
c
         do 102 i=-1,nx+2
            qw1(i) = q1(i,j,k)
            qw2(i) = q2(i,j,k)
            qw3(i) = q3(i,j,k)
            qw4(i) = q4(i,j,k)
            qw5(i) = q5(i,j,k)
            qw6(i) = q6(i,j,k)
            qw7(i) = q7(i,j,k)
 102     continue
c
         call mhdtvd
c
         do 103 i=1,nx
            q1(i,j,k) = qw1(i)
            q2(i,j,k) = qw2(i)
            q3(i,j,k) = qw3(i)
            q4(i,j,k) = qw4(i)
            q5(i,j,k) = qw5(i)
            q6(i,j,k) = qw6(i)
            q7(i,j,k) = qw7(i)
 103     continue
c
 101  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound1
c
      ncount = ncount+1
      goto 204
c-----------------------------------------------------------------------
c  integration along y
c-----------------------------------------------------------------------
 202  continue
c
      call grav
      call bound1
c
      nd = ny
      dl = dy
c
      do 111 i=1,nx
      do 111 k=1,nzp
c
         do 112 j=-1,ny+2
            qw1(j) = q1(i,j,k)
            qw2(j) = q3(i,j,k)
            qw3(j) = q4(i,j,k)
            qw4(j) = q2(i,j,k)
            qw5(j) = q6(i,j,k)
            qw6(j) = q7(i,j,k)
            qw7(j) = q5(i,j,k)
 112     continue
c
         call mhdtvd
c
         do 113 j=1,ny
            q1(i,j,k) = qw1(j)
            q3(i,j,k) = qw2(j)
            q4(i,j,k) = qw3(j)
            q2(i,j,k) = qw4(j)
            q6(i,j,k) = qw5(j)
            q7(i,j,k) = qw6(j)
            q5(i,j,k) = qw7(j)
 113     continue
c
 111  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call grav
      call bound1
c
      ncount = ncount+1
      goto 204
c-----------------------------------------------------------------------
c  integration along z
c-----------------------------------------------------------------------
 203  continue
c
      nsize = nzp*ny*nzp*7
c
      do 301 k=1,nzp
      do 301 j=1,ny
      do 301 i=1,nzp
c
         w1(i,j,k,1) = q1(pi*nzp+i,j,k)
         w1(i,j,k,2) = q2(pi*nzp+i,j,k)
         w1(i,j,k,3) = q3(pi*nzp+i,j,k)
         w1(i,j,k,4) = q4(pi*nzp+i,j,k)
         w1(i,j,k,5) = q5(pi*nzp+i,j,k)
         w1(i,j,k,6) = q6(pi*nzp+i,j,k)
         w1(i,j,k,7) = q7(pi*nzp+i,j,k)
c
 301  continue
c
      do 302 k=1,nzp
      do 302 j=1,ny
      do 302 i=1,nzp
c
         q1(pi*nzp+k,j,i) = w1(i,j,k,1)
         q2(pi*nzp+k,j,i) = w1(i,j,k,2)
         q3(pi*nzp+k,j,i) = w1(i,j,k,3)
         q4(pi*nzp+k,j,i) = w1(i,j,k,4)
         q5(pi*nzp+k,j,i) = w1(i,j,k,5)
         q6(pi*nzp+k,j,i) = w1(i,j,k,6)
         q7(pi*nzp+k,j,i) = w1(i,j,k,7)
c
 302  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      do 303 ip=1,np-1
c
         pswap = xor(pi,ip)
         ptag1 = ip
         ptag2 = 811
c
         do 304 k=1,nzp
         do 304 j=1,ny
         do 304 i=1,nzp
c
            w1(i,j,k,1) = q1(pswap*nzp+i,j,k)
            w1(i,j,k,2) = q2(pswap*nzp+i,j,k)
            w1(i,j,k,3) = q3(pswap*nzp+i,j,k)
            w1(i,j,k,4) = q4(pswap*nzp+i,j,k)
            w1(i,j,k,5) = q5(pswap*nzp+i,j,k)
            w1(i,j,k,6) = q6(pswap*nzp+i,j,k)
            w1(i,j,k,7) = q7(pswap*nzp+i,j,k)
c
 304     continue
c
         call MPI_IRECV (w2,nsize,MPI_REAL,pswap,ptag1,
     +                   MPI_COMM_WORLD,ireq(1),err)
         call MPI_IRECV (idum1,0,MPI_INTEGER,pswap,ptag2,
     +                   MPI_COMM_WORLD,ireq(2),err)
         call MPI_SEND (idum2,0,MPI_INTEGER,pswap,ptag2,
     +                  MPI_COMM_WORLD,err)
         call MPI_WAIT (ireq(2),status,err)
         call MPI_SEND (w1,nsize,MPI_REAL,pswap,ptag1,
     +                  MPI_COMM_WORLD,err)
         call MPI_WAIT (ireq(1),status,err)
c
         do 305 k=1,nzp
         do 305 j=1,ny
         do 305 i=1,nzp
c
            q1(pswap*nzp+k,j,i) = w2(i,j,k,1)
            q2(pswap*nzp+k,j,i) = w2(i,j,k,2)
            q3(pswap*nzp+k,j,i) = w2(i,j,k,3)
            q4(pswap*nzp+k,j,i) = w2(i,j,k,4)
            q5(pswap*nzp+k,j,i) = w2(i,j,k,5)
            q6(pswap*nzp+k,j,i) = w2(i,j,k,6)
            q7(pswap*nzp+k,j,i) = w2(i,j,k,7)
c
 305     continue
c
 303  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound1
c
      nd = nz
      dl = dz
c
      do 121 i=1,nzp
      do 121 j=1,ny
c
         do 122 k=-1,nz+2
            qw1(k) = q1(k,j,i)
            qw2(k) = q4(k,j,i)
            qw3(k) = q2(k,j,i)
            qw4(k) = q3(k,j,i)
            qw5(k) = q7(k,j,i)
            qw6(k) = q5(k,j,i)
            qw7(k) = q6(k,j,i)
 122     continue
c
         call mhdtvd
c
         do 123 k=1,nz
            q1(k,j,i) = qw1(k)
            q4(k,j,i) = qw2(k)
            q2(k,j,i) = qw3(k)
            q3(k,j,i) = qw4(k)
            q7(k,j,i) = qw5(k)
            q5(k,j,i) = qw6(k)
            q6(k,j,i) = qw7(k)
 123     continue
c
 121  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound1
c 
      do 311 k=1,nzp
      do 311 j=1,ny
      do 311 i=1,nzp
c
         w1(i,j,k,1) = q1(pi*nzp+k,j,i)
         w1(i,j,k,2) = q2(pi*nzp+k,j,i)
         w1(i,j,k,3) = q3(pi*nzp+k,j,i)
         w1(i,j,k,4) = q4(pi*nzp+k,j,i)
         w1(i,j,k,5) = q5(pi*nzp+k,j,i)
         w1(i,j,k,6) = q6(pi*nzp+k,j,i)
         w1(i,j,k,7) = q7(pi*nzp+k,j,i)
c
 311  continue
c
      do 312 k=1,nzp
      do 312 j=1,ny
      do 312 i=1,nzp
c
         q1(pi*nzp+i,j,k) = w1(i,j,k,1)
         q2(pi*nzp+i,j,k) = w1(i,j,k,2)
         q3(pi*nzp+i,j,k) = w1(i,j,k,3)
         q4(pi*nzp+i,j,k) = w1(i,j,k,4)
         q5(pi*nzp+i,j,k) = w1(i,j,k,5)
         q6(pi*nzp+i,j,k) = w1(i,j,k,6)
         q7(pi*nzp+i,j,k) = w1(i,j,k,7)
c
 312  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      do 313 ip=1,np-1
c
         pswap = xor(pi,ip)
         ptag1 = ip
         ptag2 = 811
c
         do 314 k=1,nzp
         do 314 j=1,ny
         do 314 i=1,nzp
c
            w1(i,j,i,1) = q1(pswap*nzp+k,j,i)
            w1(i,j,i,2) = q2(pswap*nzp+k,j,i)
            w1(i,j,i,3) = q3(pswap*nzp+k,j,i)
            w1(i,j,i,4) = q4(pswap*nzp+k,j,i)
            w1(i,j,i,5) = q5(pswap*nzp+k,j,i)
            w1(i,j,i,6) = q6(pswap*nzp+k,j,i)
            w1(i,j,i,7) = q7(pswap*nzp+k,j,i)
c
 314     continue
c
         call MPI_IRECV (w2,nsize,MPI_REAL,pswap,ptag1,
     +                   MPI_COMM_WORLD,ireq(1),err)
         call MPI_IRECV (idum1,0,MPI_INTEGER,pswap,ptag2,
     +                   MPI_COMM_WORLD,ireq(2),err)
         call MPI_SEND (idum2,0,MPI_INTEGER,pswap,ptag2,
     +                  MPI_COMM_WORLD,err)
         call MPI_WAIT (ireq(2),status,err)
         call MPI_SEND (w1,nsize,MPI_REAL,pswap,ptag1,
     +                  MPI_COMM_WORLD,err)
         call MPI_WAIT (ireq(1),status,err)
c
         do 315 k=1,nzp
         do 315 j=1,ny
         do 315 i=1,nzp
c
            q1(pswap*nzp+i,j,k) = w2(i,j,k,1)
            q2(pswap*nzp+i,j,k) = w2(i,j,k,2)
            q3(pswap*nzp+i,j,k) = w2(i,j,k,3)
            q4(pswap*nzp+i,j,k) = w2(i,j,k,4)
            q5(pswap*nzp+i,j,k) = w2(i,j,k,5)
            q6(pswap*nzp+i,j,k) = w2(i,j,k,6)
            q7(pswap*nzp+i,j,k) = w2(i,j,k,7)
c
 315     continue
c
 313  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound1
c
      ncount = ncount+1
      goto 204
c-----------------------------------------------------------------------
c  end
c-----------------------------------------------------------------------
 205  continue
c
      call MPI_BARRIER (MPI_COMM_WORLD,err)
c
      call bound2
c-----------------------------------------------------------------------
c  end
c-----------------------------------------------------------------------
      return
      end
